package crawler;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import output.DataSetOutputter;
import storers.Hotel;
import storers.Review;
import storers.Service;

/**
 * Class that crawls Booking.com
 *
 */
public class BookingComParser {

	private String urlStem = "http://www.booking.com";
	private int sleepTime = 10000;
	private String locationString = null;
	private DataSetOutputter xmloutputter = null;

	protected List<Service> search(String term) {
		locationString = term;
		xmloutputter = new DataSetOutputter("booking.com", locationString);
		Scanner in = null;
		try {
			URL url = new URL(urlStem+"/searchresults.html?src=index&error_url=http%3A%2F%2Fwww.booking.com%2Findex.en-gb.html%3Fsid%3Db484ffc04f8827707e3e115dda0bbae0%3Bdcid%3D4%3B&dcid=4&lang=en-gb&sid=b484ffc04f8827707e3e115dda0bbae0&si=ai%2Cco%2Cci%2Cre%2Cdi&ss="+term.replaceAll("\\s+", "+")+"&checkin_monthday=0&checkin_year_month=0&checkout_monthday=0&checkout_year_month=0&idf=on");
			in = new Scanner(url.openStream(), "UTF-8");
		} catch (MalformedURLException mue) {
			mue.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

		Pattern topUrlPattern = Pattern.compile("<div onclick=\"location.href=\\('(.*?)'\\);\" class=\"disambitem highlightmeboth\" >");

		String line;
		while(in.hasNextLine()) {
			line = in.nextLine().trim();
			Matcher topUrlMatcher = topUrlPattern.matcher(line);
			if(topUrlMatcher.find()) {
				System.out.println("Searching for hotels in "+term+"...");
				return readSearchResults(topUrlMatcher.group(1));
			}
		}
		in.close();
		return null;
	}

	/**
	 * Reads the search results and returns a list of hotels
	 * @param searchResults Search result
	 * @return List of hotels
	 */
	private List<Service> readSearchResults(String searchResults) {
		List<Service> hotels = new ArrayList<Service>();
		Scanner in = null;	
		int startPoint = 0;
		int endPoint = 0;
		int counter = startPoint;
		int rows = 50;
		try {
			for(int i = startPoint; i <= endPoint; i=i+rows) {
				Thread.sleep(sleepTime);
				int hotelsPerPage = 0;
				Map<String,String> urls = new HashMap<String,String>();
				
				URL url = new URL(urlStem+searchResults.replaceAll("offset=.*?;", "offset="+i+";rows="+rows+";"));
				URLConnection connection = url.openConnection();
				HttpURLConnection httpConn = (HttpURLConnection)connection;
				httpConn.addRequestProperty("User-Agent", "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11");
				InputStream is;
				if (httpConn.getResponseCode() < 400) {
					is = httpConn.getInputStream();
				} else {
					is = httpConn.getErrorStream();
					System.err.println("Error code '"+httpConn.getResponseCode()+"' returned, skipping page with offset '"+i+"'");
					is.close();
					continue;
				}
				in = new Scanner(is, "UTF-8");

				Pattern hotelPattern = Pattern.compile("<a title=\"(.*?)\" class=\"hotel_name_link url \" href=\"(.*?)\" >(.*?)</a>");

				String line = null;
				while(in.hasNextLine()) {
					line = in.nextLine().trim();
					if((line.matches("\\d{1,4}") || line.matches("\\d{1,4} hotels found in.*?")) && endPoint == 0) {
						endPoint = Integer.parseInt(line.split("hotels found in")[0].trim());
						System.out.println(endPoint+" hotels found");
						System.out.println();
					}
					Matcher hotelMatch = hotelPattern.matcher(line);
					if(hotelMatch.find()) {						
						hotelsPerPage += 1;
						urls.put(hotelMatch.group(3), hotelMatch.group(2));
					}
				}
				System.out.println(hotelsPerPage+" hotels found on this page");
				System.out.println();
				for(Entry<String,String> entry : urls.entrySet()) {
					counter += 1;
					if(!xmloutputter.serviceExists(entry.getKey())) {
						System.out.println("Hotel #"+counter+" of "+endPoint+": '"+entry.getKey()+"'");
						Hotel hotel = (Hotel)getHotel(entry.getKey(), entry.getValue());//.split("\\?")[0]);
						if(hotel==null) {
							continue;
						}
						xmloutputter.output(hotel);
						System.out.println();
					} else {
						System.out.println("Hotel #"+counter+" of "+endPoint+": '"+entry.getKey()+"' already output");
					}
				}
				is.close();
				in.close();
			}
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		} catch (MalformedURLException mue) {
			mue.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

		System.out.println(counter+" of "+endPoint+" hotels output");
		return hotels;
	}

	/**
	 * Get a hotel based on name and hotel URL
	 * @param hotelName Name of the hotel
	 * @param hotelUrl URL of the hotel
	 * @return A hotel as a Service object
	 */
	private Service getHotel(String hotelName, String hotelUrl) {

		Pattern metaPattern = Pattern.compile("<meta property=\"og:(.*?)\" content=\"(.*?)\" />");
		Pattern starPattern = Pattern.compile("<span class=\".*?\" title=\"([1-5]) star hotel\">&nbsp;</span>");
		Pattern reviewUrlPattern = Pattern.compile("<a href=.*?b25.Reviews.getRawPage\\(\'(.*?)\',.*?\">See all reviews</a>");
		Pattern facilityPattern = Pattern.compile("<span><img.*?/>(\\w{2,20})</span></h3>.*?<p.*?>(.*?)</p>");
		Pattern reviewAmountPattern = Pattern.compile("<strong class=\"count\">(.*?)</strong> reviews");
		String address = null;
		String latitude = null;
		String longitude = null;
		String street = null;
		String locality = null;
		String postal = null;
		String region = null;
		String stars = null;
		String reviewUrl = null;
		int reviewAmount = 0;
		String facGeneral = null;
		String facActivities = null;
		String facServices = null;
		String facInternet = null;
		String facParking = null;

		try {
			Thread.sleep(sleepTime);
			System.out.println("Getting info and reviews for hotel '"+hotelName+"'");
			URL url = new URL(urlStem+hotelUrl);
			Scanner in = new Scanner(url.openStream(), "UTF-8");

			String line = "";
			while(in.hasNextLine()) {
				line += in.nextLine().trim();
			}

			Matcher metaMatch = metaPattern.matcher(line);
			while(metaMatch.find()) {
				if(metaMatch.group(1).equals("street-address")) {
					street = metaMatch.group(2);
				} else if (metaMatch.group(1).equals("locality")) {
					locality = metaMatch.group(2);
				} else if (metaMatch.group(1).equals("region")) {
					region = metaMatch.group(2);
				} else if (metaMatch.group(1).equals("postal-code")) {
					postal = metaMatch.group(2);
				} else if (metaMatch.group(1).equals("latitude")) {
					latitude = metaMatch.group(2);
				} else if (metaMatch.group(1).equals("longitude")) {
					longitude = metaMatch.group(2);
				}
			}

			Matcher reviewAmountMatcher = reviewAmountPattern.matcher(line);
			if(reviewAmountMatcher.find()) {
				reviewAmount = Integer.parseInt(reviewAmountMatcher.group(1));
			}
			
			Matcher starMatcher = starPattern.matcher(line);
			if(starMatcher.find()) {
				stars = starMatcher.group(1);
			}

			Matcher reviewMatcher = reviewUrlPattern.matcher(line);
			if(reviewMatcher.find()) {
				reviewUrl = reviewMatcher.group(1);
			}
			
			Matcher facilityMatcher = facilityPattern.matcher(line);
			while(facilityMatcher.find()) {
				String facilityType = facilityMatcher.group(1);
				String facilityContent = facilityMatcher.group(2).replaceAll("<.*?>", " ").trim();
				if(facilityType.equals("General")) {
					facGeneral = facilityContent;
				} else if(facilityType.equals("Activities")) {
					facActivities = facilityContent;
				} else if(facilityType.equals("Services")) {
					facServices = facilityContent;
				} else if(facilityType.equals("Internet")) {
					facInternet = facilityContent;
				} else if(facilityType.equals("Parking")) {
					facParking = facilityContent;
				}  
			}

			address = street+", "+region+", "+locality+", "+postal;

			in.close();
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		} catch (MalformedURLException mue) {
			mue.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		if(stars==null) {
			stars = "0";
		}
		Service hotel = new Hotel(hotelName, address, Integer.parseInt(stars));
		hotel.setLatitude(Double.parseDouble(latitude));
		hotel.setLongitude(Double.parseDouble(longitude));
		System.out.print("Finding reviews..");
		if(reviewAmount > 0) {
			hotel.setReviews(getReviews(reviewUrl, reviewAmount));
		} else {
			hotel.setReviews(new ArrayList<Review>());
			System.out.print("no reviews found");
			System.out.println();
		}
		hotel.setFacilities(facGeneral, facActivities, facServices, facInternet, facParking);
		if(hotel.getReviews()==null) {
			return null;
		}
		return hotel;
	}

	/**
	 * Get all the reviews based on the URL
	 * @param reviewsUrl Review URL
	 * @param reviewAmount Number of reviews
	 * @return Return a list of reviews
	 */
	private List<Review> getReviews(String reviewsUrl, int reviewAmount) {
		List<Review> reviews = new ArrayList<Review>();
		boolean englishReviews = true;
		Scanner in = null;
		int counter = 0;
		int endPoint = reviewAmount;

		Pattern reviewPattern = Pattern.compile("<tr id=\"(\\d+?)\" class=\"back_.*?review_tr\">(.*?)</tr>");

		Pattern contentPattern = Pattern.compile("<div class=\"cell_user_name\" style=\"padding-bottom: 3px;\">(.*?)</div>.*?<div class=\"cell_user_profile\">(.*?)</div>.*?<span class=\"country-name\">(.*?)</span>.*?<span class=\"cell_user_date\">(.*?)</span>");
		Pattern commentPattern = Pattern.compile("<p class=\"comments_(.*?)\" lang=\"(.*?)\">(.*?)</p>");
		Pattern helpfulPattern = Pattern.compile("<span class=\"vote_copy\">(\\d{1,9}) of \\d{1,9}.*?</span>");
		Pattern scorePattern = Pattern.compile("<div class=\"cell_score_shade_and_value\">(.*?)</div>");
		Pattern scorePattern2 = Pattern.compile("<td class=\"cell_score\">.*?<span>(.*?)</span>.*?</td>");

		System.out.print(endPoint+" reviews found");
		System.out.println();
		
		try {
			for(int i = 0; i <= endPoint; i=i+100) {
				if(englishReviews==false) {
					break;
				}				
				Thread.sleep(sleepTime);
				reviewsUrl = reviewsUrl.replaceAll("offset=.*?;", "offset="+i+";").replace(";rid=", "").trim();
				URL url = new URL(urlStem+reviewsUrl+";rows=100");

				URLConnection connection = url.openConnection();
				HttpURLConnection httpConn = (HttpURLConnection)connection;
				httpConn.addRequestProperty("User-Agent", "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11");
				InputStream is;
				if (httpConn.getResponseCode() < 400) {
					is = httpConn.getInputStream();
				} else {
					is = httpConn.getErrorStream();
				}
				in = new Scanner(is);

				String line = null;
				String input = "";
				while(in.hasNextLine()) {
					line = in.nextLine().trim();

					if(line.matches("\\(Total (\\d{1,4})\\)") && endPoint == 0) {
						endPoint = Integer.parseInt(line.replaceAll("[^0-9]", ""));
					}

					input += " "+line;
				}

				Matcher reviewMatch = reviewPattern.matcher(input);
				while(reviewMatch.find()) {
					String username = null;
					String profile = null;
					String country = null;
					DateTime date = null;
					String commentGood = null;
					String commentBad = null;
					String helpful = null;
					String score = null;
					String lang = null;
					Matcher contentMatch = contentPattern.matcher(reviewMatch.group(2));
					if(contentMatch.find()) {
						username = contentMatch.group(1).trim();
						profile = contentMatch.group(2).trim();
						country = contentMatch.group(3).trim();
						date = createDate(contentMatch.group(4));
						if(username.equals("")) {
							username = "Anonymous";
						}
					} else {
						continue;
					}
					Matcher commentMatch = commentPattern.matcher(reviewMatch.group(2));
					while(commentMatch.find()) {
						if(lang==null) {
							lang = commentMatch.group(2);
						}
						if(commentMatch.group(1).equals("good")) {
							commentGood = commentMatch.group(3);
						} else if(commentMatch.group(1).equals("bad")) {
							commentBad = commentMatch.group(3);
						}
					}
					Matcher helpfulMatch = helpfulPattern.matcher(reviewMatch.group(2));
					if(helpfulMatch.find()) {
						helpful = helpfulMatch.group(1);
					}
					Matcher scoreMatch = scorePattern.matcher(reviewMatch.group(2));
					if(scoreMatch.find()) {
						score = scoreMatch.group(1);
					} else {
						scoreMatch = scorePattern2.matcher(reviewMatch.group(2));
						if(scoreMatch.find()) {
							score = scoreMatch.group(1);
						}
					}

					if(lang != null && lang.equals("en") && username != null) {
						Review review = new Review();
						counter += 1;
						review.setUsername(username);
						review.setProfile(profile);
						review.setCountry(country);
						review.setDate(date);
						if(commentGood != null) {
							review.setCommentGood(commentGood);
						}
						if(commentBad != null) {
							review.setCommentBad(commentBad);
						}
						if(helpful != null) {
							review.setHelpfulScore(Integer.parseInt(helpful));
						}
						review.setScore(Double.parseDouble(score));
						reviews.add(review);
					} else {
						englishReviews = false;
					}
				}
				System.out.print(counter+"..");
				is.close();
				in.close();
			}
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		} catch (MalformedURLException mue) {
			mue.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		
		System.out.println();
		System.out.println(counter+" English reviews found, of "+endPoint+" total");
		
		if(counter==0) {
			return null; 
		}
		return reviews;
	}

	/**
	 * Creates date
	 * @param dateString Date as String object
	 * @return Date as DateTime object
	 */
	private DateTime createDate(String dateString) {
		DateTimeFormatter dtf = null;
		if(dateString.matches("\\d{1,2}\\s\\w{3,12}\\s\\d{4}")) {
			dtf = DateTimeFormat.forPattern("dd MMMM yyyy").withZone(DateTimeZone.UTC);
		} else if(dateString.matches("\\w{3,12}\\s\\d{1,2},\\s\\d{4}")) {
			dtf = DateTimeFormat.forPattern("MMMM dd, yyyy").withZone(DateTimeZone.UTC);
		} else {
			return null;
		}
		return dtf.parseDateTime(dateString);
	}
}
