package gatherer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import storers.HotelScore;

/**
 * Class that reads average score from XML file from [source]/averages/
 *
 */
public class ReadAverageScore {

	private List<HotelScore> hs;

	public ReadAverageScore(){
		hs = new ArrayList<HotelScore>();
	}

	/**
	 * Gets the score from XML
	 * @param location Location
	 * @param source Source
	 */
	public void getScoresFromXML(String location, String source){
		String filename = source + "/averages/"+location+".xml";
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(new File(filename));
			Element root = doc.getRootElement();
			List<Element> hotel = root.getChildren();
			double lastScoreSentiment = 0.0;
			double lastActualScore = 0.0;
			for (int i = 0; i<hotel.size(); i++){
				HotelScore hotelScore = new HotelScore(hotel.get(i).getAttributeValue("name"), hotel.get(i).getAttributeValue("address"), 
						Double.parseDouble(hotel.get(i).getAttributeValue("latitude")), Double.parseDouble(hotel.get(i).getAttributeValue("longitude"))); 
				List<Element> actualSentiment = hotel.get(i).getChildren();
				for (int j = 0; j<actualSentiment.size(); j++){
					if (actualSentiment.get(j).getName().equals("sentiment")){
						List<Element> monthlyTotal = actualSentiment.get(j).getChildren();
						for (int l = 0; l<monthlyTotal.size(); l++){
							if (monthlyTotal.get(l).getName() == "total"){
								hotelScore.setTotalReviews(Integer.parseInt(monthlyTotal.get(l).getChild("score").getAttributeValue("reviews")));
								hotelScore.setTotalScore(Double.parseDouble(monthlyTotal.get(l).getChildText("score")));
								continue;
							}
							List<Element> score = monthlyTotal.get(l).getChildren();
							for (int k = 0; k<score.size(); k++){
								if (score.get(k).getParentElement().getName().equals("monthly")){
									hotelScore.setChangeMonthlyScore(Integer.parseInt(score.get(k).getAttributeValue("year")+score.get(k).getAttributeValue("month")), Double.parseDouble(score.get(k).getText())-lastScoreSentiment);
									hotelScore.setAverageMonthlyScore(Integer.parseInt(score.get(k).getAttributeValue("year")+score.get(k).getAttributeValue("month")), 
											Double.parseDouble(score.get(k).getText()));
									hotelScore.setNumberMonthlyReviews(Integer.parseInt(score.get(k).getAttributeValue("year")+score.get(k).getAttributeValue("month")), Integer.parseInt(score.get(k).getAttributeValue("reviews")));
									lastScoreSentiment = Double.parseDouble(score.get(k).getText());
								}
							}
						}
					}
					if (actualSentiment.get(j).getName().equals("actual")){
						List<Element> monthlyTotal = actualSentiment.get(j).getChildren();
						for (int l = 0; l<monthlyTotal.size(); l++){
							if (monthlyTotal.get(l).getName() == "total"){
								//hotelScore.setTotalReviews(Integer.parseInt(monthlyTotal.get(l).getChild("score").getAttributeValue("reviews")));
								hotelScore.setTotalActualScore(Double.parseDouble(monthlyTotal.get(l).getChildText("score")));
								continue;
							}
							List<Element> score = monthlyTotal.get(l).getChildren();
							for (int k = 0; k<score.size(); k++){
								if (score.get(k).getParentElement().getName().equals("monthly")){
									hotelScore.setChangeActualScore(Integer.parseInt(score.get(k).getAttributeValue("year")+score.get(k).getAttributeValue("month")), Double.parseDouble(score.get(k).getText())-lastActualScore);
									hotelScore.setAverageActualScore(Integer.parseInt(score.get(k).getAttributeValue("year")+score.get(k).getAttributeValue("month")), 
											Double.parseDouble(score.get(k).getText()));	
									lastActualScore = Double.parseDouble(score.get(k).getText());
								}
							}
						}
					}
				}
				hs.add(hotelScore);
			}
		} catch(FileNotFoundException e) {
			System.err.println("File '"+filename+"' not found. Please choose a different source or location.");
		} catch(IOException e) {
			e.printStackTrace();
		} catch(JDOMException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args){
		String source = "tripadvisor";
		String[] locations = {"london", "new york city", "dubai", "athens", "paris"};

		ArrayList<Double> counter = new ArrayList<Double>();
		double a = 0;
		double b = 0;
		double c = 0;
		double d = 0;
		double e = 0;
		double f = 0;
		double g = 0;
		double h = 0;
		double j = 0;
		double k = 0;
		double l = 0;
		double tot = 0;

		File dir = new File(source+"/averages");
		for(String file : dir.list()) {
			System.out.println(file);
			boolean loc = false;
			for(String str : locations) {
				if(str.equals(file.split("\\.")[0])) {
					loc = true;
					break;
					//		ReadAverageScore ras= new ReadAverageScore();
					//		ras.getScoresFromXML("paris", "tripadvisor");
					//		for (int i = 0; i<ras.getHs().size(); i++){
					//			//System.out.println("Name: "+ ras.getHs().get(i).getName());
					//			HashMap<Integer, Integer> numberReviews = ras.getHs().get(i).getNumberMonthlyReviews();
					//			//System.out.println("Address: " + ras.getHs().get(i).getAddress());
					//			//System.out.println("Latitude/longitude: " + ras.getHs().get(i).getLatitude() + "/" + ras.getHs().get(i).getLongitude());
					//			for(Map.Entry<Integer, Double> entry : ras.getHs().get(i).getChangeMonthlyScore().entrySet()){
					//				if ((entry.getValue() >= 0.2 || entry.getValue() <= -0.2) && numberReviews.get(entry.getKey()) >= 7){
					//					System.out.println(numberReviews.get(entry.getKey()));
					//					System.out.println("Average Change Sentiment Score: " + entry.getKey() + " " + entry.getValue());
					//					System.out.println();
					//					//Sjekke for stabilitet i forandringene?
					//				}
				}
				if(!loc) continue;
				ReadAverageScore ras= new ReadAverageScore();
				ras.getScoresFromXML(file.split("\\.")[0], source);

				System.out.println(ras.getHs().size()+" services found");

				for(HotelScore hs : ras.getHs()) {
					double actual = hs.getTotalActualScore();
					double sentiment = hs.getTotalScore();
					counter.add(sentiment);
				}
				for(double i : counter) {
					tot += 1;
					if(i < 0.1) {
						a += 1;
					} else if(i < 0.2) {
						b += 1;
					} else if(i < 0.3) {
						c += 1;
					} else if(i < 0.4) {
						d += 1;
					} else if(i < 0.5) {
						e += 1;
					} else if(i < 0.6) {
						f += 1;
					} else if(i < 0.7) {
						g += 1;
					} else if(i < 0.8) {
						h += 1;
					} else if(i < 0.9) {
						j += 1;
					} else if(i < 10) {
						k += 1;
					} else {
						l += 1;
						System.out.println(i);
					}
				}
			}

			System.out.println("<1 "+((double)a/tot));
			System.out.println("1-2 "+((double)b/tot));
			System.out.println("2-3 "+((double)c/tot));
			System.out.println("3-4 "+((double)d/tot));
			System.out.println("4-5 "+((double)e/tot));
			System.out.println("5-6 "+((double)f/tot));
			System.out.println("6-7 "+((double)g/tot));
			System.out.println("7-8 "+((double)h/tot));
			System.out.println("8-9 "+((double)j/tot));
			System.out.println("9-10 "+((double)k/tot));
			System.out.println(">10 "+((double)l/tot));
		}


	}

	public List<HotelScore> getHs() {
		return hs;
	}

	public void setHs(List<HotelScore> hs) {
		this.hs = hs;
	}
}
