package output;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.jdom.Element;
import org.jdom.output.XMLOutputter;

import processing.FeatureExtraction;

import storers.Sentence;
import storers.Service;

/**
 * Class that outputs the average score as xml in [source]/averages
 *
 */
public class FeatureOutputter {

	private Element locationNode = null;
	private List<Service> services = null;
	FeatureExtraction fe = null;

	public FeatureExtraction getFeatureExtraction() {
		return fe;
	}

	public FeatureOutputter(String source, String location) {
		fe = new FeatureExtraction(source, location);
		fe.extractFeatures(source, location);
		setServices(fe.getReadDataSet().getServices());
		locationNode = new Element("location");
		locationNode.setAttribute("name", fe.getReadDataSet().getLocation());
		startFeatureOutputter(source, location);
	}

	/**
	 * Creates a node for the service
	 * @param service Service, typically Hotel
	 * @return service node
	 */
	private Element createServiceNode(Service service) {
		Element serviceNode = new Element(service.getType().toString().toLowerCase());
		serviceNode.setAttribute("name", service.getName());
		serviceNode.setAttribute("pos_score", String.valueOf(service.getTotalPosScore()));
		serviceNode.setAttribute("neg_score", String.valueOf(service.getTotalNegScore()));
		serviceNode.setAttribute("number_features", String.valueOf(service.getTotalNumberFeatures()));
		serviceNode.setAttribute("lat", String.valueOf(service.getLatitude()));
		serviceNode.setAttribute("lon", String.valueOf(service.getLongitude()));
		return serviceNode;
	}

	/**
	 * Calculate average sentiment score
	 * @param service Service
	 * @param serviceNode Service node
	 */
	private void createFeatureNode(Service service, Element serviceNode) {
		for (int i = 0; i < fe.getFeatureWords().length; i++){
			Element featureNode = new Element("features");
			featureNode.setAttribute("name", fe.getFeatureWords()[i]);
			featureNode.setAttribute("pos_score", String.valueOf(service.getFeaturePosScores()[i]));
			featureNode.setAttribute("neg_score", String.valueOf(service.getFeatureNegScores()[i]));
			featureNode.setAttribute("number", String.valueOf(service.getTotalNumbers()[i]));
			for(Sentence sentence : service.getSentences()) {
				if (sentence.getFeature().equals(fe.getFeatureWords()[i])){
					Element sentenceElement = new Element("sentence");
					sentenceElement.setAttribute("pos_score", String.valueOf(sentence.getPosScore()));
					sentenceElement.setAttribute("neg_score", String.valueOf(sentence.getNegScore()));
					sentenceElement.setText(sentence.getSentence());
					featureNode.addContent(sentenceElement);
				}
			}
			serviceNode.addContent(featureNode);
		}
	}

	private void addServiceNode(Element serviceNode) {
		locationNode.addContent(serviceNode);
	}

	/**
	 * Writes to file.
	 */
	public void write(boolean comma){
		try {
			XMLOutputter fmt = new XMLOutputter();
			FileOutputStream fos;
			if (comma){
				fos = new FileOutputStream(fe.getReadDataSet().getSource()+"/features/"+fe.getReadDataSet().getLocation()+"_splitcomma.xml");
			}
			else {
				fos = new FileOutputStream(fe.getReadDataSet().getSource()+"/features/"+fe.getReadDataSet().getLocation()+".xml");
			}
			fmt.output(locationNode, fos);
			if (comma){
				System.out.println("File '"+fe.getReadDataSet().getSource()+"/features/"+fe.getReadDataSet().getLocation()+"_splitcomma.xml' updated");
			}
			else {
				System.out.println("File '"+fe.getReadDataSet().getSource()+"/features/"+fe.getReadDataSet().getLocation()+".xml' updated");
			}
		} catch(IOException e) {
			e.printStackTrace();
		}
	}

	public void startFeatureOutputter(String source, String location) {
		for(Service service : getServices()) {
			Element serviceNode = createServiceNode(service);
			createFeatureNode(service, serviceNode);
			addServiceNode(serviceNode);
		}
		write(false);
	}

	public List<Service> getServices() {
		return services;
	}

	public void setServices(List<Service> services) {
		this.services = services;
	}
}
