package output;

import com.aliasi.tokenizer.IndoEuropeanTokenizerFactory;
import com.aliasi.tokenizer.TokenizerFactory;
import com.aliasi.util.Files;

import com.aliasi.classify.BernoulliClassifier;
import com.aliasi.classify.BinaryLMClassifier;
import com.aliasi.classify.Classification;
import com.aliasi.classify.Classified;
import com.aliasi.classify.DynamicLMClassifier;
import com.aliasi.classify.NaiveBayesClassifier;

import com.aliasi.lm.NGramProcessLM;
import com.aliasi.lm.TokenizedLM;

import java.io.File;
import java.io.IOException;

/*
 * Class for performing machine learning through the LingPipe library. Supports Dynamic LM Classifier and Naive Bayes 
 * Classifier
 */
public class MachineLearning {

    File mPolarityDir;
    String[] mCategories;
    DynamicLMClassifier<NGramProcessLM> mClassifier;
    //NaiveBayesClassifier mClassifier;

    MachineLearning(String[] args) {
        System.out.println("\nBASIC POLARITY DEMO");
        mPolarityDir = new File("test");
        System.out.println("\nData Directory=" + mPolarityDir);
        mCategories = mPolarityDir.list();
        int nGram = 8;
        mClassifier 
        	//= new NaiveBayesClassifier(mCategories, new IndoEuropeanTokenizerFactory(), nGram, 128);
        	= DynamicLMClassifier.createNGramProcess(mCategories,nGram);
    }

    /*
     * Runs the training and evaluation methods
     */
    void run() throws ClassNotFoundException, IOException {
        train();
        evaluate();
    }

    /*
     * Checks if the file is a training file
     */
    boolean isTrainingFile(File file) {
        return !(Integer.parseInt(file.getName().replace(".txt", "")) <= 2000);  // test on fold 9
    }

    /*
     * Method for performing the training
     */
    void train() throws IOException {
        int numTrainingCases = 0;
        int numTrainingChars = 0;
        System.out.println("\nTraining.");
        for (int i = 0; i < mCategories.length; ++i) {
            String category = mCategories[i];
            Classification classification
                = new Classification(category);
            File file = new File(mPolarityDir,mCategories[i]);
            File[] trainFiles = file.listFiles();
            for (int j = 0; j < trainFiles.length; ++j) {
                File trainFile = trainFiles[j];
                if (isTrainingFile(trainFile)) {
                    ++numTrainingCases;
                    String review = Files.readFromFile(trainFile,"ISO-8859-1");
                    numTrainingChars += review.length();
                    Classified<CharSequence> classified
                        = new Classified<CharSequence>(review,classification);
                    mClassifier.handle(classified);
                }
            }
        }
        System.out.println("  # Training Cases=" + numTrainingCases);
        System.out.println("  # Training Chars=" + numTrainingChars);
    }

    /*
     * Method for evaluation to see the degree of correctness
     */
    void evaluate() throws IOException {
        System.out.println("\nEvaluating.");
        int numTests = 0;
        int numCorrect = 0;
        for (int i = 0; i < mCategories.length; ++i) {
            String category = mCategories[i];
            File file = new File(mPolarityDir,mCategories[i]);
            File[] trainFiles = file.listFiles();
            for (int j = 0; j < trainFiles.length; ++j) {
                File trainFile = trainFiles[j];
                if (!isTrainingFile(trainFile)) {
                    String review = Files.readFromFile(trainFile,"ISO-8859-1");
                    ++numTests;
                    Classification classification
                        = mClassifier.classify(review);
                    if (classification.bestCategory().equals(category))
                        ++numCorrect;
                }
            }
        }
        System.out.println("  # Test Cases=" + numTests);
        System.out.println("  # Correct=" + numCorrect);
        System.out.println("  % Correct=" 
                           + ((double)numCorrect)/(double)numTests);
    }

    public static void main(String[] args) {
        try {
            new MachineLearning(args).run();
        } catch (Throwable t) {
            System.out.println("Thrown: " + t);
            t.printStackTrace(System.out);
        }
    }

}
