package output;

import gatherer.ReadAverageScore;
import gatherer.ReadDataSet;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import processing.PosTagger;

import storers.Hotel;
import storers.HotelScore;
import storers.Review;
import storers.Service;

/**
 * Class that performs simple burst detection of the changes in the average monthly semantic score 
 *
 */
public class SimpleBurstDetection {

	private int limitNumberReviews = 10;
	private double limitChangeScore = 0.2;
	private ArrayList<Hotel> hotels;
	private String[] stopWordList = {"was", "were", "have", "has", "having", "is", "are", "be", "do", "had"};

	/**
	 * Constructor
	 */
	public SimpleBurstDetection(){
		hotels = new ArrayList<Hotel>();
	}

	/**
	 * Method that performs burst detection
	 * @param source Name of the source, typically "tripadvisor" or "booking.com"
	 * @param location Location of the place one wants to perform burst detection
	 */
	public void burstDetection(String source, String location){
		int correct = 0;
		int total = 0;
		ReadAverageScore ras= new ReadAverageScore();
		ras.getScoresFromXML(location, source);
		ReadDataSet rds = new ReadDataSet(source, location);
		rds.read();
		DateTimeFormatter dtf = DateTimeFormat.forPattern("yyyyMM").withZone(DateTimeZone.UTC);
		for (int i = 0; i<ras.getHs().size(); i++){
			HashMap<Integer, Integer> numberReviews = ras.getHs().get(i).getNumberMonthlyReviews();
			String name = ras.getHs().get(i).getName();
			String address = ras.getHs().get(i).getAddress();
			Hotel hotel = new Hotel(name, address, 0.0);
			hotel.setLatitude(ras.getHs().get(i).getLatitude());
			hotel.setLongitude(ras.getHs().get(i).getLongitude());
			boolean addReviews = false;
			HashMap<Integer, Double> changeMonthlyScore = ras.getHs().get(i).getChangeMonthlyScore();
			for(Map.Entry<Integer, Double> entry : ras.getHs().get(i).getChangeMonthlyScore().entrySet()){
				DateTime date = dtf.parseDateTime(String.valueOf(entry.getKey()));
				DateTime lastMonth = date.minusMonths(1);
				DateTime nextMonth = date.plusMonths(1);
				int monthLast = lastMonth.getMonthOfYear();
				int yearLast = lastMonth.getYear();
				int monthNext = nextMonth.getMonthOfYear();
				int yearNext = nextMonth.getYear();
				if (!(numberReviews.containsKey(Integer.parseInt(yearLast + "" + monthLast))) || !(numberReviews.containsKey(Integer.parseInt(yearNext + "" + monthNext)))){
					continue;
				}

				if ((entry.getValue() >= limitChangeScore || entry.getValue() <= limitChangeScore*-1) && (numberReviews.get(entry.getKey()) >= limitNumberReviews) && (numberReviews.get(Integer.parseInt(yearLast + "" + monthLast)) >= limitNumberReviews) && 
						((changeMonthlyScore.get(Integer.parseInt(yearNext + "" + monthNext)) <= 0.1) && (changeMonthlyScore.get(Integer.parseInt(yearNext + "" + monthNext)) >= -0.1))){
//				if (name.equals("Grand Midwest Express Hotel Apartments") && entry.getKey() == 20117){	
//					System.out.println("Next month: " + changeMonthlyScore.get(Integer.parseInt(yearNext + "" + monthNext)));
//					System.out.println("Last month: " + changeMonthlyScore.get(Integer.parseInt(yearLast + "" + monthLast)));
					List<Service> l = rds.getServices();
					if (checkCorrectness(ras.getHs().get(i), entry.getValue(), entry.getKey())){
						correct++;
						total++;
					}
					else {
						total++;
					}
					System.out.println("Correct " + correct + "/" + total);
					for (int j = 0; j < l.size(); j++){
						if (l.get(j).getName().equals(name)){
							List<Review> r = l.get(j).getReviews();
							List<Review> reviews = new ArrayList<Review>();
							for (int k = 0; k < r.size(); k++){
								if ((r.get(k).getDate().getMonthOfYear() == date.getMonthOfYear()) && (r.get(k).getDate().getYear() == date.getYear())){
									addReviews = true;
									reviews.add(r.get(k));
								}
							}
							hotel.setReviews(reviews);
						}
					}
					System.out.println("Average Change Sentiment Score for hotel " + name + " : " + entry.getKey() + " " + entry.getValue());
					System.out.println();
				}
			}
			if (addReviews == true){
				hotels.add(hotel);
			}
		}
	}

	public ArrayList<Hotel> getHotels() {
		return hotels;
	}

	/**
	 * Checks for correctness
	 * @param hs The hotel score
	 * @param value Value
	 * @param key Key
	 * @return
	 */
	public boolean checkCorrectness(HotelScore hs, Double value, Integer key){
		System.out.println("ChangeActualScore: " + hs.getChangeActualScore().get(key));
		double limitChangeScoreHalf = 0.05;
		if (value > 0.0){
			if (hs.getChangeActualScore().get(key) > limitChangeScoreHalf){
				return true;
			}
			else {
				return false;
			}
		}
		else if (value < 0.0){
			if (hs.getChangeActualScore().get(key) < limitChangeScoreHalf*-1){
				return true;
			}
			else {
				return false;
			}
		}
		else {
			return false;
		}
	}
	
	public void runSimpleBurstDetection(String source, String location){
		burstDetection(source, location);
		PosTagger postagger = new PosTagger();
		HashMap<String, Integer> groupWordsWithNumbers;

		for (int i = 0; i<hotels.size(); i++){
			System.out.println();
			System.out.println("Name: " + hotels.get(i).getName());
			System.out.println("Lat, Long: " + hotels.get(i).getLatLong());
			groupWordsWithNumbers = new HashMap<String, Integer>();
			for (int j = 0; j < hotels.get(i).getReviews().size(); j++){
				postagger.tagReviewWordGroups(hotels.get(i).getReviews().get(j));
				System.out.println(hotels.get(i).getReviews().get(j).getDate().toString());
				System.out.println("Score: " + hotels.get(i).getReviews().get(j).getScore());
								//System.out.println("Pos score: " + sbd.hotels.get(i).getReviews().get(j).getPosScore());
								//System.out.println("Neg score: " + sbd.hotels.get(i).getReviews().get(j).getNegScore());
				//System.out.println(sbd.hotels.get(i).getReviews().get(j).getAllComments());
								//for (Entry<String, Integer> e : sbd.hotels.get(i).getReviews().get(j).getTaggedWords().entrySet()){
								//	System.out.println("Words " + e.getKey() + " " + e.getValue());
								//}
				for (int k = 0; k < hotels.get(i).getReviews().get(j).getGroupWordsSummary().size(); k++){
					//System.out.println(sbd.hotels.get(i).getReviews().get(j).getGroupWordsSummary().get(k));
					String wordPair = hotels.get(i).getReviews().get(j).getGroupWordsSummary().get(k);
					if (groupWordsWithNumbers.containsKey(wordPair)){
						groupWordsWithNumbers.put(wordPair, groupWordsWithNumbers.get(wordPair) + 1);
					}
					else {
						groupWordsWithNumbers.put(wordPair, 1);
					}
				}
			}
			for (Entry<String, Integer> e : groupWordsWithNumbers.entrySet()){
				boolean foundStopWord = false;
				if (e.getValue() > 0){
					for (int n = 0; n < stopWordList.length; n++){
						if (stopWordList[n].equals(e.getKey().split(" ")[1])){
							foundStopWord = true;
							break;
						}
					}
					if (foundStopWord == false){
						System.out.println(e.getKey() + " " + e.getValue());
					}
				}
			}
			groupWordsWithNumbers = null;
			System.out.println();
		}
	}
}
