package processing;

import edu.stanford.nlp.ling.CoreAnnotations.SentencesAnnotation;
import gatherer.ReadDataSet;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.sun.org.apache.xerces.internal.impl.xpath.regex.RegularExpression;

import output.ScoreOutputter;
import storers.Review;
import storers.Sentence;
import storers.SentiWord;
import storers.Service;

/**
 * Class for extracting features from sentences. 
 *
 */
public class FeatureExtraction {

	private String[] featureWords = {"breakfast", "staff", "service", "clean", "location", "internet"};

	private ReadDataSet readerDataSet = null;
	private SentiWordNet swn;

	public FeatureExtraction(String source, String location){
		readerDataSet = new ReadDataSet(source, location);
		swn = new SentiWordNet();
		swn.addSentiWords();
	}

/**
 * Calculates scores for each sentence in a review
 * @param review Review
 * @param sentenceID Sentence ID
 */
	public void scoreAnalyzer(Review review, int sentenceID, WordNet wn){
		//SentiWordNet swn = new SentiWordNet();
		//swn.addSentiWords();
		HashMap<String, SentiWord> sws = swn.getSentiWords();

		double totalPos = 0.0;
		double totalNeg = 0.0;

		ArrayList<String> taggedSentences = review.getTaggedSentences();
		totalPos = 0.0;
		totalNeg = 0.0;
		String[] wordsAndWordTypesTemp = {};
		try {
			wordsAndWordTypesTemp = taggedSentences.get(sentenceID).split("\\s+");
		}
		catch(IndexOutOfBoundsException e){
			System.out.println("Sentence ID: " + sentenceID);
			System.out.println("TaggedSentences.size(): " + taggedSentences.size());
			for (int i = 0; i<taggedSentences.size(); i++){
				System.out.println(taggedSentences.get(i));
			}
			for (int j = 0; j < review.getRegularSentence().size(); j++){
				System.out.println(review.getRegularSentence().get(j));
			}
			System.out.println(review.getAllComments());
			System.out.println("Exception!");
		}
		for (int o = 0; o < wordsAndWordTypesTemp.length; o++){
			String[] wordsAndWordTypes = wordsAndWordTypesTemp[o].split("_");
			for (int k = 0; k < wordsAndWordTypes.length-1; k=k+2){
				String id = wn.getID(wordsAndWordTypes[k], wordsAndWordTypes[k+1]);
				if (id == null){
					continue;
				}
				SentiWord sw = sws.get(id+wordsAndWordTypes[k+1]);
				totalPos += sw.getPosValue();
				totalNeg += sw.getNegValue();
			}
		}
		review.setSentencePosScore(sentenceID, totalPos);
		review.setSentenceNegScore(sentenceID, totalNeg);
	}
	
	/**
	 * Method that extracts features
	 * @param source
	 * @param location
	 */
	public void extractFeatures(String source, String location){
		PosTagger postagger = new PosTagger();
		readerDataSet.read();
		WordNet wn = new WordNet();
		wn.openDictionary();
		for (int i = 0; i < readerDataSet.getServices().size(); i++){
			System.out.println("Service number " + (i+1) + "/" + readerDataSet.getServices().size());
			double posTotalScore = 0.0;
			double negTotalScore = 0.0;
			int totalNumberFeatures = 0;
			double[] posFeatureScores = new double[6];
			double[] negFeatureScores = new double[6];
			int[] totalNumbers = new int[6];
			ArrayList<Sentence> sentences = new ArrayList<Sentence>();
			for (int j = 0; j< readerDataSet.getServices().get(i).getReviews().size(); j++){
//				ArrayList<String> taggedSentences = readerDataSet.getServices().get(i).getReviews().get(j).getTaggedSentences();
				if (readerDataSet.getServices().get(i).getReviews().get(j).getRegularSentence().size() == 0){
					postagger.createRegularSentence(readerDataSet.getServices().get(i).getReviews().get(j));
				}
				ArrayList<String> regularSentences = readerDataSet.getServices().get(i).getReviews().get(j).getRegularSentence();
				for (int k = 0; k < regularSentences.size(); k++){
					boolean checkedBefore = false;
					for (int l = 0; l < featureWords.length; l++){
						if (regularSentences.get(k).contains(featureWords[l])){
							if (checkedBefore == false){
								postagger.tagSentence(readerDataSet.getServices().get(i).getReviews().get(j));
							}
							checkedBefore = true;
							scoreAnalyzer(readerDataSet.getServices().get(i).getReviews().get(j), k, wn);
							Sentence sentence = new Sentence(k, readerDataSet.getServices().get(i).getReviews().get(j).getRegularSentence().get(k), featureWords[l], readerDataSet.getServices().get(i).getReviews().get(j).getSentencePosScore().get(k), readerDataSet.getServices().get(i).getReviews().get(j).getSentenceNegScore().get(k));
							sentences.add(sentence);
							posFeatureScores[l] += readerDataSet.getServices().get(i).getReviews().get(j).getSentencePosScore().get(k);
							negFeatureScores[l] += readerDataSet.getServices().get(i).getReviews().get(j).getSentenceNegScore().get(k);
							totalNumbers[l]++;
							posTotalScore += readerDataSet.getServices().get(i).getReviews().get(j).getSentencePosScore().get(k);
							negTotalScore += readerDataSet.getServices().get(i).getReviews().get(j).getSentenceNegScore().get(k);
							totalNumberFeatures++;
						}
					}
				}
				readerDataSet.getServices().get(i).setSentences(sentences);
				readerDataSet.getServices().get(i).setFeaturePosScores(posFeatureScores);
				readerDataSet.getServices().get(i).setFeatureNegScores(negFeatureScores);
				readerDataSet.getServices().get(i).setTotalNumberFeatures(totalNumberFeatures);
				readerDataSet.getServices().get(i).setTotalNumbers(totalNumbers);
				readerDataSet.getServices().get(i).setTotalNegScore(negTotalScore);
				readerDataSet.getServices().get(i).setTotalPosScore(posTotalScore);
//				for (int p = 0; p < readerDataSet.getServices().get(i).getSentences().size(); p++){
//					System.out.println(readerDataSet.getServices().get(i).getSentences().get(p).getSentence());
//				}
				//System.out.println(totalScore);
				//System.out.println(totalNumberFeatures);
				//System.out.println();
			}
		}
	}

	public static void main(String[] args){
		FeatureExtraction fe = new FeatureExtraction("tripadvisor", "hammerfest");
		fe.extractFeatures("tripadvisor", "hammerfest");
	}

	public ReadDataSet getReadDataSet() {
		return readerDataSet;
	}
	
	public String[] getFeatureWords() {
		return featureWords;
	}
}
