package processing;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import storers.Review;

import edu.stanford.nlp.tagger.maxent.MaxentTagger;
import edu.stanford.nlp.tagger.maxent.TaggerConfig;
import gatherer.ReadDataSet;

/**
 * Class that performs the pos tagging.
 *
 */
public class PosTagger {

	private static final Pattern posPattern = Pattern.compile(" pos=\"(.*?)\"");
	private static final Pattern wordPattern = Pattern.compile("<word .*?>(.*?)</word>");
	private static final Pattern sentencePattern = Pattern.compile("<sentence id=.*?>(.*?)</sentence>");
	private MaxentTagger tagger;
	TaggerConfig config;

	/**
	 * Constructor that adds the tagging model and outputformat.
	 */
	public PosTagger(){
		String modelFile = "models/english-left3words-distsim.tagger";
		String outputFormat = "xml";
		config = new TaggerConfig("-model", modelFile, "-outputFormat", outputFormat);
		try {
			tagger = new MaxentTagger(config.getModel(), config);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Tags all the reviews based on source and location
	 * @param source Source, typically "booking.com" or "tripadvisor"
	 * @param location name of location
	 * @return All the reviews tagged in a ReadDataSet
	 */
	public ReadDataSet tagReviews(String source, String location){
		ReadDataSet reader = new ReadDataSet(source, location);
		reader.read();
		for (int i = 0; i < reader.getServices().size(); i++){
			System.out.println(i);
			for (int j = 0; j < reader.getServices().get(i).getReviews().size(); j++){
				Review review = reader.getServices().get(i).getReviews().get(j);
				tagReview(review);
			}
		}
		return reader;
	}
	
	public ReadDataSet tagSentences(String source, String location){
		ReadDataSet reader = new ReadDataSet(source, location);
		reader.read();
		for (int i = 0; i < reader.getServices().size(); i++){
			System.out.println(i);
			for (int j = 0; j < reader.getServices().get(i).getReviews().size(); j++){
				Review review = reader.getServices().get(i).getReviews().get(j);
				tagSentence(review);
			}
		}
		return reader;
	}

	/**
	 * Tags individual reviews
	 * @param review Review one wants to tag
	 */
	public void tagReview(Review review){
		InputStream is = new ByteArrayInputStream(review.getAllComments().getBytes());
		BufferedReader in = new BufferedReader(new InputStreamReader(is));
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter("output.txt"));
			BufferedReader in2;
			String line = "";
			Matcher matchPos;
			Matcher matchWord;
			String pos = null;
			String word = null;
			tagger.runTagger(in, out, "", true);
			in2 = new BufferedReader(new FileReader(new File("output.txt")));
			while (( line = in2.readLine()) != null){
				matchPos = posPattern.matcher(line);
				pos = null;
				word = null;
				if(matchPos.find()){
					pos = matchPos.group(1);
				}
				matchWord = wordPattern.matcher(line);
				if (matchWord.find()){
					word = matchWord.group(1).toLowerCase();
				}
				if (pos == null || word == null){
					continue;
				}
				//			if (pos.equals("UH")){
				//				teller++;
				//				System.out.println("Antall injections: " + teller);
				//				System.out.println("Ord: " + word);
				//			}
				if (pos.equals("RB") || pos.equals("RBS") || pos.equals("RBR")){
					pos = "r"; //Adverbs 
				}
				if (pos.equals("NN") || pos.equals("NNP") || pos.equals("NNS") || pos.equals("NNPS")){
					pos = "n"; //Nouns
				}
				if (pos.equals("JJ") || pos.equals("JJR") || pos.equals("JJS")){
					pos = "a"; //Adjectives
				}
				if (pos.equals("VB") || pos.equals("VBD") || pos.equals("VBG") || pos.equals("VBN") || pos.equals("VBP") || pos.equals("VBZ")){
					pos = "v"; //Verbs
				}
				if (!(pos.equals("r")) && !(pos.equals("n")) && !(pos.equals("a")) && !(pos.equals("v"))){
					continue;
				}
				if (word.equals("n't")){
					word = "not";
				}
				if (word.equals("'s") && pos.equals("v")){
					word = "is";
				}
				if (word.equals("'m") && pos.equals("v")){
					word = "am";
				}
				word += "_"+pos;
				review.setTaggedWord(word, 1); //Increments the value in the set method if the key exist
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Tags individual reviews
	 * @param review Review one wants to tag
	 */
	public void tagSentence(Review review){
		InputStream is = new ByteArrayInputStream(review.getAllComments().getBytes());
		BufferedReader in = new BufferedReader(new InputStreamReader(is));
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter("output.txt"));
			BufferedReader in2;
			String line = "";
			Matcher matchPos;
			Matcher matchWord;
			String pos = null;
			String word = null;
			String sentence = "";
			tagger.runTagger(in, out, "", true);
			in2 = new BufferedReader(new FileReader(new File("output.txt")));
			while (( line = in2.readLine()) != null){
				matchPos = posPattern.matcher(line);
				pos = null;
				word = null;
				if(matchPos.find()){
					pos = matchPos.group(1);
				}
				matchWord = wordPattern.matcher(line);
				if (matchWord.find()){
					word = matchWord.group(1).toLowerCase();
				}
				if (pos == null || word == null){
					continue;
				}
				//			if (pos.equals("UH")){
				//				teller++;
				//				System.out.println("Antall injections: " + teller);
				//				System.out.println("Ord: " + word);
				//			}
				if (pos.equals("RB") || pos.equals("RBS") || pos.equals("RBR")){
					pos = "r"; //Adverbs 
				}
				if (pos.equals("NN") || pos.equals("NNP") || pos.equals("NNS") || pos.equals("NNPS")){
					pos = "n"; //Nouns
				}
				if (pos.equals("JJ") || pos.equals("JJR") || pos.equals("JJS")){
					pos = "a"; //Adjectives
				}
				if (pos.equals("VB") || pos.equals("VBD") || pos.equals("VBG") || pos.equals("VBN") || pos.equals("VBP") || pos.equals("VBZ")){
					pos = "v"; //Verbs
				}
				if (!(pos.equals("r")) && !(pos.equals("n")) && !(pos.equals("a")) && !(pos.equals("v")) && !(pos.equals("."))){
					continue;
				}
				if (word.equals("n't")){
					word = "not";
				}
				if (word.equals("'s") && pos.equals("v")){
					word = "is";
				}
				if (word.equals("'m") && pos.equals("v")){
					word = "am";
				}
				word += "_"+pos;
				
				if (pos.equals(".")){
					review.setTaggedSentences(sentence);
					//System.out.println("Tag sentence: " + sentence);
					//sentence = sentence.replaceAll("_[a-z]", "");
					//review.setRegularSentence(sentence);
					sentence = "";
				}
				else {
					sentence += word + " ";
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}
	
	public void createRegularSentence(Review review){
		InputStream is = new ByteArrayInputStream(review.getAllComments().getBytes());
		BufferedReader in = new BufferedReader(new InputStreamReader(is));
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter("output.txt"));
			BufferedReader in2;
			String line = "";
			Matcher matchPos;
			Matcher matchWord;
			String pos = null;
			String word = null;
			String sentence = "";
			tagger.runTagger(in, out, "", true);
			in2 = new BufferedReader(new FileReader(new File("output.txt")));
			while (( line = in2.readLine()) != null){
				matchPos = posPattern.matcher(line);
				pos = null;
				word = null;
				if(matchPos.find()){
					pos = matchPos.group(1);
				}
				matchWord = wordPattern.matcher(line);
				if (matchWord.find()){
					word = matchWord.group(1).toLowerCase();
				}
				if (pos == null || word == null){
					continue;
				}
				//			if (pos.equals("UH")){
				//				teller++;
				//				System.out.println("Antall injections: " + teller);
				//				System.out.println("Ord: " + word);
				//			}

				if (word.equals("n't")){
					word = "not";
				}
				if (word.equals("'s") && pos.equals("v")){
					word = "is";
				}
				if (word.equals("'m") && pos.equals("v")){
					word = "am";
				}
				if (word.equals("-lrb-") || word.equals("-rrb-") || word.equals("''") || word.equals("``")){
					continue;
				}
				
				if (pos.equals(".")){
					if (!sentence.equals("")){
						review.setRegularSentence(sentence.trim());
						//System.out.println("Regular sentence: " + sentence);
					}
					sentence = "";
				}
				else {
					if (pos.equals(";") || pos.equals("'") || word.equals("'s") || pos.equals(":") || pos.equals(",")){
						sentence += word;
					}
					else {
						sentence += " " + word;	
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Tags reviews in different groups. Performs simple feature extraction. 
	 * @param review Review one wants to tag.
	 */
	public void tagReviewWordGroups(Review review){
		InputStream is = new ByteArrayInputStream(review.getAllComments().getBytes());
		BufferedReader in = new BufferedReader(new InputStreamReader(is));
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter("output.txt"));
			BufferedReader in2;
			String line = "";
			Matcher matchPos;
			Matcher matchWord;
			String pos = null;
			String word = null;
			tagger.runTagger(in, out, "", true);
			String lastAdjective = null;
			String lastAdverb = null;
			in2 = new BufferedReader(new FileReader(new File("output.txt")));
			while (( line = in2.readLine()) != null){
				matchPos = posPattern.matcher(line);
				pos = null;
				word = null;
				if(matchPos.find()){
					pos = matchPos.group(1);
				}
				matchWord = wordPattern.matcher(line);
				if (matchWord.find()){
					word = matchWord.group(1).toLowerCase();
				}
				if (pos == null || word == null){
					continue;
				}
				//			if (pos.equals("UH")){
				//				teller++;
				//				System.out.println("Antall injections: " + teller);
				//				System.out.println("Ord: " + word);
				//			}
				if (word.equals("n't")){
					word = "not";
				}
				if (word.equals("'s") && pos.equals("v")){
					word = "is";
				}
				if (word.equals("'m") && pos.equals("v")){
					word = "am";
				}
				
				if (pos.equals("RB") || pos.equals("RBS") || pos.equals("RBR")){
					pos = "r"; //Adverbs 
					lastAdverb = word;
				}
				if (pos.equals("NN") || pos.equals("NNP") || pos.equals("NNS") || pos.equals("NNPS")){
					pos = "n"; //Nouns
					if (lastAdjective != null){
						review.setGroupWordsSummary(lastAdjective + " " + word);
					}
					lastAdjective = null;
				}
				if (pos.equals("JJ") || pos.equals("JJR") || pos.equals("JJS")){
					pos = "a"; //Adjectives
					lastAdjective = word;
				}
				if (pos.equals("VB") || pos.equals("VBD") || pos.equals("VBG") || pos.equals("VBN") || pos.equals("VBP") || pos.equals("VBZ")){
					pos = "v"; //Verbs
					if (lastAdverb != null){
						review.setGroupWordsSummary(lastAdverb + " " + word);
					}
					lastAdverb = null;
				}
				if (!(pos.equals("r")) && !(pos.equals("n")) && !(pos.equals("a")) && !(pos.equals("v"))){
					continue;
				}
				word += "_"+pos;
				review.setTaggedWord(word, 1); //Increments the value in the set method if the key exist
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}
	
}