package processing;

import java.util.ArrayList;
import java.util.List;

import com.aliasi.spell.JaroWinklerDistance;

import output.DataSetOutputter;

import storers.Hotel;
import storers.Review;
import storers.Service;

import gatherer.ReadDataSet;

/**
 * Class that merges Booking.com and TripAdvisor data.
 *
 */
public class ServiceCombiner {
	
	private ReadDataSet bookingData;
	private ReadDataSet tripadvData;
	
	private List<Service> bookingServices;
	private List<Service> tripadvServices;
	private List<Service> mergedList;
	
	private String location;
	
	
	public ServiceCombiner(String location) {
		this.location = location;
		bookingData = new ReadDataSet("booking.com", location);
		tripadvData = new ReadDataSet("tripadvisor", location);
		bookingData.read();
		tripadvData.read();
		
		bookingServices = bookingData.getServices();
		tripadvServices = tripadvData.getServices();
		mergedList = new ArrayList<Service>();
	}
	
	private void output() {
		DataSetOutputter outputter = new DataSetOutputter("merged", location);
		outputter.output(mergedList);
	}
	
	private void merge() {
		for(Service bs : bookingServices) {
			String bsName = stringCleaner(bs.getName());
			double closestValue = 0;
			Service closest = null;

			for(Service ts : tripadvServices) {
				String tsName = stringCleaner(ts.getName());
				double prox = jaroWinklerDistance(bsName, tsName);
				if(prox > closestValue) {
					closestValue = prox;
					closest = ts;
				}
			}
			if(closestValue > 0.9) {
				mergedList.add(mergeServices(bs, closest));
			} else if(closestValue > 0.8) {
				float dist = distFrom(bs.getLatitude(), bs.getLongitude(), closest.getLatitude(), closest.getLongitude());
				String bAddr = bs.getAddress().split("[,|]")[0];
				String cAddr = closest.getAddress().split("[,|]")[0];
				double addrMatch = jaroWinklerDistance(bAddr, cAddr);
				if(dist < 500 && addrMatch > 0.9) {
					mergedList.add(mergeServices(bs, closest));
				}
			}
		}
		System.out.println(bookingServices.size());
		System.out.println(tripadvServices.size());
		System.out.println(mergedList.size());
	}
	
	/*
	 * Merges two services into one. Service A is dominant and takes precedent when selecting fields
	 */
	private Service mergeServices(Service A, Service B) {
		Service temp = new Hotel(A.getName(), A.getAddress(), B.getStars());
		List<Review> reviews = A.getReviews();
		reviews.addAll(B.getReviews());
		temp.setReviews(reviews);
		temp.setFacilities(A.getFacilities());
		if(A.getLatitude() != 0.0 && A.getLongitude() != 0.0) {
			temp.setLatitude(A.getLatitude());
			temp.setLongitude(A.getLongitude());
		} else {
			temp.setLatitude(B.getLatitude());
			temp.setLongitude(B.getLongitude());
		}
		return temp;
	}

	private float distFrom(double lat1, double lng1, double lat2, double lng2) {
	    double earthRadius = 3958.75;
	    double dLat = Math.toRadians(lat2-lat1);
	    double dLng = Math.toRadians(lng2-lng1);
	    double a = Math.sin(dLat/2) * Math.sin(dLat/2) +
	               Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2)) *
	               Math.sin(dLng/2) * Math.sin(dLng/2);
	    double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
	    double dist = earthRadius * c;

	    int meterConversion = 1609;

	    return (float) (dist * meterConversion);
	}
	
	private String stringCleaner(String str) {
		str = str.toLowerCase();
		str = str.replace("&amp;", " and ");
		str = str.replaceAll("&.{1,4};", "");
		str = str.replaceAll("[^a-zA-Z0-9]", " ");
		str = str.replaceAll("\\s+", " ");
		
		//Remove Domain specific terms
		str = str.replace("hotel", "");
		str = str.replace("apartments", "");
		str = str.replace(location, "");
		str = str.replace("the", "");
		return str.toLowerCase().trim();
	}
	
	private double jaroWinklerDistance(String A, String B) {
		JaroWinklerDistance jaroWinkler = JaroWinklerDistance.JARO_WINKLER_DISTANCE;	
		return jaroWinkler.proximity(A,B);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		ServiceCombiner sc = new ServiceCombiner("dubai");
		sc.merge();
		sc.output();
	}
}
