package storers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Class that contains information about a hotel including all its reviews. 
 *
 */
public class Hotel extends Service {

	private String name;
	private String address;
	private double grade;
	private ServiceType type = ServiceType.HOTEL;
	private List<Review> reviews = new ArrayList<Review>();
	private double latitude;
	private double longitude;
	private String general;
	private String activities;
	private String services;
	private String internet;
	private String parking;
	private double totalPosScore;
	private double totalNegScore;
	private int totalNumberFeatures;
	private double[] featurePosScores;
	private double[] featureNegScores;
	private int[] totalNumbers;
	private HashMap<String, String> facilities;
	private ArrayList<Sentence> sentences;

	public Hotel(String name, String address, double stars) {
		this.name = name;
		this.address = address;
		this.grade = stars;
		this.latitude = 0.0;
		this.longitude = 0.0;
		this.facilities = null;
		this.setTotalNegScore(0.0);
		this.setTotalPosScore(0.0);
		this.setTotalNumberFeatures(0);
		this.setFeaturePosScores(new double[6]);
		this.setFeatureNegScores(new double[6]);
		this.setTotalNumbers(new int[6]);
		sentences = new ArrayList<Sentence>();
	}
	
	@Override
	public List<Review> getReviews() {
		return this.reviews;
	}
	
	@Override
	public void addReview(Review review) {
		this.reviews.add(review);
	}
	
	@Override
	public ServiceType getType() {
		return this.type;
	}

	@Override
	public String getName() {
		return this.name;
	}

	@Override
	public String getAddress() {
		return this.address;
	}

	@Override
	public double getStars() {
		return this.grade;
	}

	@Override
	public void setReviews(List<Review> reviews) {
		this.reviews = reviews;
	}
	
	public void setLatitude(double latitude) {
		this.latitude = latitude;
	}
	public void setLongitude(double longitude) {
		this.longitude = longitude;
	}
	
	public String getLatLong() {
		return this.latitude+", "+this.longitude;
	}

	@Override
	public double getLatitude() {
		return this.latitude;
	}

	@Override
	public double getLongitude() {
		return this.longitude;
	}
	
	public HashMap<String, String> getFacilities() {
		return facilities;
	}
	public void setFacilities(Map<String,String> facilities) {
		this.facilities = (HashMap<String,String>)facilities;
	}
	
	public void setFacilities(String facGeneral, String facActivities,
			String facServices, String facInternet, String facParking) {
		
		this.facilities = new HashMap<String,String>();
		if(facGeneral != null) {
			this.facilities.put("general", facGeneral);
		}
		if(facActivities != null) {
			this.facilities.put("activities", facActivities);
		}
		if(facServices != null) {
			this.facilities.put("services", facServices);
		}
		if(facInternet != null) {
			this.facilities.put("internet", facInternet);
		}
		if(facParking != null) {
			this.facilities.put("parking", facParking);
		}
	}

	public int[] getTotalNumbers() {
		return totalNumbers;
	}

	public void setTotalNumbers(int[] totalNumbers) {
		this.totalNumbers = totalNumbers;
	}

	public double[] getFeaturePosScores() {
		return featurePosScores;
	}
	
	public double[] getFeatureNegScores() {
		return featureNegScores;
	}

	public void setFeaturePosScores(double[] featureScores) {
		this.featurePosScores = featureScores;
	}
	
	public void setFeatureNegScores(double[] featureScores) {
		this.featureNegScores = featureScores;
	}

	public int getTotalNumberFeatures() {
		return totalNumberFeatures;
	}

	public void setTotalNumberFeatures(int totalNumberFeatures) {
		this.totalNumberFeatures = totalNumberFeatures;
	}

	public ArrayList<Sentence> getSentences() {
		return sentences;
	}

	public void setSentences(ArrayList<Sentence> sentences) {
		this.sentences = sentences;
	}

	public void setTotalPosScore(double totalScore) {
		totalPosScore = totalScore;
		
	}

	public double getTotalPosScore() {
		return totalPosScore;
	}

	public void setTotalNegScore(double totalScore) {
		totalNegScore = totalScore;
		
	}

	public double getTotalNegScore() {
		return totalNegScore;
	}
}
