package storers;

import java.util.ArrayList;
import java.util.HashMap;

import org.joda.time.DateTime;

/**
 * Review class that contains all the information about a review.
 *
 */
public class Review implements Comparable<Review> {

	private String username;
	private int reviewCount;
	private int totalHelpfulScore; 
	private DateTime date;
	private String country;
	private String profile;
	private String comment;
	private String commentGood;
	private String commentBad;
	private int helpfulScore;
	private double score;
	private double posScore;
	private double negScore;
	private HashMap<String, Integer> taggedWords;
	private ArrayList<String> sentences;
	private ArrayList<String> taggedSentences;
	private ArrayList<String> groupWordsSummary;
	private ArrayList<String> regularSentence;
	private HashMap<Integer, Double> sentencePosScore;
	private HashMap<Integer, Double> sentenceNegScore;

	public Review() {
		this.username = null;
		this.reviewCount = 1;
		this.totalHelpfulScore = 0;
		this.date = null;
		this.country = null;
		this.profile = null;
		this.comment = null;
		this.commentGood = null;
		this.commentBad = null;
		this.helpfulScore = 0;
		this.score = 0;
		this.taggedWords = new HashMap<String, Integer>();
		taggedSentences = new ArrayList<String>();
		sentences = new ArrayList<String>();
		groupWordsSummary = new ArrayList<String>();
		sentencePosScore = new HashMap<Integer, Double>();
		regularSentence = new ArrayList<String>();
		sentenceNegScore = new HashMap<Integer, Double>();
		
	}
	
	public String getUsername() {
		return username;
	}
	public void setUsername(String username) {
		this.username = username;
	}
	public DateTime getDate() {
		return date;
	}
	public void setDate(DateTime date) {
		this.date = date;
	}
	public String getCountry() {
		return country;
	}
	public void setCountry(String country) {
		this.country = country;
	}
	public String getProfile() {
		return profile;
	}
	public void setProfile(String profile) {
		this.profile = profile;
	}
	public String getComment() {
		return comment;
	}
	public void setComment(String comment) {
		this.comment = comment;
	}
	public String getCommentGood() {
		return commentGood;
	}
	public void setCommentGood(String commentGood) {
		this.commentGood = commentGood;
	}
	public String getCommentBad() {
		return commentBad;
	}
	public void setCommentBad(String commentBad) {
		this.commentBad = commentBad;
	}
	public int getHelpfulScore() {
		return helpfulScore;
	}
	public void setHelpfulScore(int helpfulScore) {
		this.helpfulScore = helpfulScore;
	}
	public double getScore() {
		return score;
	}
	public void setScore(double score) {
		this.score = score;
	}

	public int getReviewCount() {
		return reviewCount;
	}
	public void setReviewCount(int reviewCount) {
		this.reviewCount = reviewCount;
	}
	public int getTotalHelpfulScore() {
		return totalHelpfulScore;
	}
	public void setTotalHelpfulScore(int totalHelpfulScore) {
		this.totalHelpfulScore = totalHelpfulScore;
	}

	public void setComment(String comment, String sentiment) {
		if(sentiment.toLowerCase().equals("good")) {
			this.setCommentGood(comment);
		} else if(sentiment.toLowerCase().equals("bad")) {
			this.setCommentBad(comment);
		} else {
			this.setComment(comment);
		}
	}
	
	public String getAllComments() {
		if(this.comment==null) {
			this.comment = "";
		}
		if(this.commentGood==null) {
			this.commentGood = "";
		}
		if(this.commentBad==null) {
			this.commentBad = "";
		}
		return (this.comment+" "+this.commentGood+" "+this.commentBad).trim();
	}
	
	public void setTaggedWord(String key, Integer value){
		if (taggedWords.containsKey(key)){
			Integer temp = taggedWords.get(key);
			temp++;
			taggedWords.put(key, temp);
		}
		else {
			taggedWords.put(key, value);
		}
	}
	
	public HashMap<String, Integer> getTaggedWords(){
		return taggedWords;
	}

	public double getPosScore() {
		return posScore;
	}

	public void setPosScore(double posScore) {
		this.posScore = posScore;
	}

	public double getNegScore() {
		return negScore;
	}

	public void setNegScore(double negScore) {
		this.negScore = negScore;
	}
	
	/**
	 * Compares whether the review date is before, after or equal to another review.
	 */
	public int compareTo(Review review) {
		final int BEFORE = -1;
		final int EQUAL = 0;
		final int AFTER = 1;
		
		if(this.getDate().isBefore(review.getDate())) {
			return BEFORE;
		} else if(this.getDate().isAfter(review.getDate())) {
			return AFTER;
		}
		return EQUAL;
	}
	
	public ArrayList<String> getGroupWordsSummary() {
		return groupWordsSummary;
	}

	public void setGroupWordsSummary(String groupWordsSummary) {
		this.groupWordsSummary.add(groupWordsSummary);
	}

	public ArrayList<String> getSentences() {
		return sentences;
	}

	public void setSentences(String sentence) {
		sentences.add(sentence);
	}
	
	/**
	 * Splits text based on sentences
	 * @return Sentences
	 */
	public ArrayList<String> splitSentences(){
		String[] temp = getAllComments().split("[.]|[!]");
		for (int i = 0; i<temp.length; i++){
			if (!(temp[i].equals(""))){
				temp[i] = temp[i].replaceAll("&quot;", "");
				sentences.add(temp[i].trim().toLowerCase());
			}
		}
		return sentences;
	}

	public ArrayList<String> getTaggedSentences() {
		return taggedSentences;
	}

	public void setTaggedSentences(String sentences) {
	//	if (!(sentences.equals(""))){
			taggedSentences.add(sentences);
	//	}
	}

	public HashMap<Integer, Double> getSentencePosScore() {
		return sentencePosScore;
	}

	public void setSentencePosScore(int id, double score) {
		this.sentencePosScore.put(id, score);
	}

	public ArrayList<String> getRegularSentence() {
		return regularSentence;
	}

	public void setRegularSentence(String sentence) {
		if (!(sentence.equals(""))){
			regularSentence.add(sentence);	
		}
	}

	public HashMap<Integer, Double> getSentenceNegScore() {
		return sentenceNegScore;
	}

	public void setSentenceNegScore(int id, double score) {
		this.sentenceNegScore.put(id, score);
	}
}
