package ui;

import java.util.Scanner;

import output.AverageScoreOutputter;
import output.FeatureOutputter;
import output.ScoreOutputter;
import output.SimpleBurstDetection;

import crawler.ParserInterface;

/**
 * Simple command line interface.
 *
 */
public class CommandLine {

	public static void main(String[] args){
		Scanner in = new Scanner(System.in); 
		Scanner in2;
		String sourceLocation;

		System.out.println("1: Crawl TripAdvisor or Booking.com");
		System.out.println("2: Write feature XML files");		
		System.out.println("3: Write score XML files");
		System.out.println("4: Write average score XML files");
		System.out.println("5: Run burst detection");
		System.out.println("0: Quit"); 
		boolean quit = false; 
		int menuItem; 
		do { 
			System.out.print("Choose menu item: "); 
			menuItem = in.nextInt(); 
			switch (menuItem) { 
			case 1: 
				System.out.println("Type in source and location. Example \"tripadvisor london\" or \"booking.com athens\"."); 
				ParserInterface pi = new ParserInterface();
				in2 = new Scanner(System.in);
				sourceLocation = in2.nextLine().trim();
				pi.startParserInterface(sourceLocation.substring(0, sourceLocation.indexOf(" ")).trim(), sourceLocation.substring(sourceLocation.indexOf(" ")).trim());
				break; 
			case 2: 
				System.out.println("Type in source and location. Example \"tripadvisor london\" or \"booking.com athens\".");
				in2 = new Scanner(System.in);
				sourceLocation = in2.nextLine().trim();
				FeatureOutputter fo = new FeatureOutputter(sourceLocation.substring(0, sourceLocation.indexOf(" ")).trim(), sourceLocation.substring(sourceLocation.indexOf(" ")).trim());
				break; 
			case 3:
				System.out.println("Type in source and location. Running the Lesk algorithm is optional. Example \"tripadvisor london lesk\" or \"booking.com athens\".");
				in2 = new Scanner(System.in);
				sourceLocation = in2.nextLine().toLowerCase().trim();
				ScoreOutputter so = new ScoreOutputter();
				if (sourceLocation.endsWith("lesk")){
					System.out.println("Running lesk, source '"+sourceLocation.substring(0, sourceLocation.indexOf(" ")).trim()+"', location '"+sourceLocation.substring(sourceLocation.indexOf(" "), sourceLocation.length()-4).trim()+"'");
					so.runScoreOutputter(sourceLocation.substring(0, sourceLocation.indexOf(" ")).trim(), sourceLocation.substring(sourceLocation.indexOf(" "), sourceLocation.length()-4).trim(), "lesk");
				}
				else {
					System.out.println("Running normal, source '"+sourceLocation.substring(0, sourceLocation.indexOf(" ")).trim()+"', location '"+sourceLocation.substring(sourceLocation.indexOf(" ")).trim()+"'");
					so.runScoreOutputter(sourceLocation.substring(0, sourceLocation.indexOf(" ")).trim(), sourceLocation.substring(sourceLocation.indexOf(" ")).trim(), "");
				}
			case 4:
				System.out.println("Type in source. Example \"tripadvisor\" or \"booking.com\".");
				in2 = new Scanner(System.in);
				sourceLocation = in2.nextLine().trim();
				AverageScoreOutputter aso = new AverageScoreOutputter();
				aso.runAverageScoreOutputter(sourceLocation);
			case 5:
				System.out.println("Type in source and location. Example \"merged london\" or \"booking.com athens\".");
				in2 = new Scanner(System.in);
				sourceLocation = in2.nextLine().trim();
				SimpleBurstDetection sbd = new SimpleBurstDetection();
				sbd.runSimpleBurstDetection(sourceLocation.substring(0, sourceLocation.indexOf(" ")).trim(), sourceLocation.substring(sourceLocation.indexOf(" ")).trim());
			case 0: 
				quit = true; 
				break; 
			default: 
				System.out.println("Invalid choice."); 
			} 
		} while (!quit); 
		System.out.println("Bye-bye!"); 
	}
}
